
/*With the sketch: "testforinstructable" you can test at least the output values of the calculation algorithms.
the entry is annotated.
Enter the day, the month, the year (on 2 digits) and number of day in the month ( like january 31 feb 28)
First enter the longitude with the sign - if the longitude is "EAST"
(east of the Greenwich meridian) degree, minute, second.
Then enter the latitude, degree, minute, second.


modify line 317

*/









#include <math.h>

//* fonction principale :

void calculerEphemeride(int jour, int mois, int annee, double longitude_ouest, double latitude_nord, double *lever, double *meridien, double *coucher);
//Entrées :
//   jour
//   mois
//   annee : valeur 00 à 99 ou bien 2000 à 2099
//   longitude_ouest : nombre décimal, négatif si longitude est
//   latitude_nord : nombre décimal, négatif si latitude sud
//Sorties : lever, meridien, coucher sous forme de nombre décimal (julien)
//   -0.5 =>  0h00 UTC
//    0.0 => 12h00 UTC
//    0.5 => 24h00 UTC
//
//Les valeurs obtenues peuvent être vérifiées sur le site de l'Institut de Mécanique Céleste et de Calcul des Ephémérides (IMCCE)
//http://www.imcce.fr
//=> Ephémérides
//=> Phénomènes célestes
//=> Levers, couchers et passages au méridien des corps du système solaire
//=> Lieu géographique à saisir en coordonnées latitude / longitude
//=> Options : précision = seconde, Format = Sexagésimal


//* fonction interne :

void calculerCentreEtVariation(double longitude_ouest, double sinlat, double coslat, double d, double *centre, double *variation)
{
  //constantes précalculées par le compilateur
  const double M_2PI = 2.0 * M_PI;
  const double degres = 180.0 / M_PI;
  const double radians = M_PI / 180.0;
  const double radians2 = M_PI / 90.0;
  const double m0 = 357.5291;
  const double m1 = 0.98560028;
  const double l0 = 280.4665;
  const double l1 = 0.98564736;
  const double c0 = 0.01671;
  const double c1 = degres * (2.0*c0 - c0*c0*c0/4.0);
  const double c2 = degres * c0*c0 * 5.0 / 4.0;
  const double c3 = degres * c0*c0*c0 * 13.0 / 12.0;
  const double r1 = 0.207447644182976; // = tan(23.43929 / 180.0 * M_PI / 2.0)
  const double r2 = r1*r1;
  const double d0 = 0.397777138139599; // = sin(23.43929 / 180.0 * M_PI)
  const double o0 = -0.0106463073113138; // = sin(-36.6 / 60.0 * M_PI / 180.0)

  double M,C,L,R,dec,omega,x;

  //deux ou trois petites formules de calcul
  M = radians * fmod(m0 + m1 * d, 360.0);
  C = c1*sin(M) + c2*sin(2.0*M) + c3*sin(3.0*M);
  L = fmod(l0 + l1 * d + C, 360.0);
  x = radians2 * L;
  R = -degres * atan((r2*sin(x))/(1+r2*cos(x)));
  *centre = (C + R + longitude_ouest)/360.0;

  dec = asin(d0*sin(radians*L));
  omega = (o0 - sin(dec)*sinlat)/(cos(dec)*coslat);
  if ((omega > -1.0) && (omega < 1.0))
    *variation = acos(omega) / M_2PI;
  else
    *variation = 0.0;
}

void calculerEphemeride(int jour, int mois, int annee, double longitude_ouest, double latitude_nord, double *lever, double *meridien, double *coucher)
{
  int nbjours;
  const double radians = M_PI / 180.0;
  double d, x, sinlat, coslat;

  //calcul nb jours écoulés depuis le 01/01/2000
  if (annee > 2000) annee -= 2000;
  nbjours = (annee*365) + ((annee+3)>>2) + jour - 1;
  switch (mois)
  {
    case  2 : nbjours +=  31; break;
    case  3 : nbjours +=  59; break;
    case  4 : nbjours +=  90; break;
    case  5 : nbjours += 120; break;
    case  6 : nbjours += 151; break;
    case  7 : nbjours += 181; break;
    case  8 : nbjours += 212; break;
    case  9 : nbjours += 243; break;
    case 10 : nbjours += 273; break;
    case 11 : nbjours += 304; break;
    case 12 : nbjours += 334; break;
  }
  if ((mois > 2) && (annee&3 == 0)) nbjours++;
  d = nbjours;

  //calcul initial meridien & lever & coucher
  x = radians * latitude_nord;
  sinlat = sin(x);
  coslat = cos(x);
  calculerCentreEtVariation(longitude_ouest, sinlat, coslat, d + longitude_ouest/360.0, meridien, &x);
  *lever = *meridien - x;
  *coucher = *meridien + x;

  //seconde itération pour une meilleure précision de calcul du lever
  calculerCentreEtVariation(longitude_ouest, sinlat, coslat, d + *lever, lever, &x);
  *lever = *lever - x;

  //seconde itération pour une meilleure précision de calcul du coucher
  calculerCentreEtVariation(longitude_ouest, sinlat, coslat, d + *coucher, coucher, &x);
  *coucher = *coucher + x;
}

//************************************************************************************************
//* exemple d'utilisation
//************************************************************************************************

#ifdef Arduino_h //sur plateforme arduino

  void initialiserAffichage()
  {
    Serial.begin(9600);
  }

  void afficherTexte(char texte[])
  {
    Serial.print(texte);
  }

#else //sur plateforme PC

  #include <stdio.h>

  void initialiserAffichage()
  {
  }

  void afficherTexte(char texte[])
  {
    printf(texte);
  }

  void setup();

  int main()
  {
    setup();
    return (int) 0;
  }

  #include <time.h>

  unsigned long int millis()
  {
      return (unsigned long int) clock() * 1000 / CLOCKS_PER_SEC;
  }

#endif

void afficherDate(int jour, int mois, int annee)
{
  char texte[20];
  sprintf(texte, "%02d/%02d/%04d", jour, mois, annee);
  afficherTexte(texte);
}

void afficherCoordonnee(int degre, int minute, int seconde, char c)
{
  char texte[20];
  if (degre < 0)
  {
    degre = -degre;
    if (c == 'W') c = 'E';
    if (c == 'N') c = 'S';
  }
  sprintf(texte,"%dd%d'%d''%c", degre, minute, seconde, c);
  afficherTexte(texte);
}

void afficherHeure(double d)
{
  int h,m,s;
  char texte[20];

  d = d + 0.5;

  if (d < 0.0)
  {
    afficherTexte("(J-1)");
    d = d + 1.0;
  }
  else
  {
    if (d > 1.0)
    {
      afficherTexte("(J+1)");
      d = d - 1.0;
    }
    else
    {
      afficherTexte("     ");
    }
  }

  h = d * 24.0;
  d = d - (double) h / 24.0;
  m = d * 1440.0;
  d = d - (double) m / 1440.0;
  s = d * 86400.0 + 0.5;

  sprintf(texte,"%02d:%02d:%02d",h,m,s);
  afficherTexte(texte);
}

double calculerCoordonneeDecimale(int degre, int minute, int seconde)
{
  if (degre > 0)
  {
    return (double) degre + minute / 60.0 + seconde / 3600.0;
  }
  else
  {
    return (double) degre - minute / 60.0 - seconde / 3600.0;
  }
}

void avancerDate(int *jour, int *mois, int *annee)
{
  (*jour)++;
  if ((*jour>31)
  || ((*jour==31) && (*mois==2 || *mois==4 || *mois==6 || *mois==9 || *mois==11))
  || ((*jour==30) && (*mois==2))
  || ((*jour==29) && (*mois==2) && (((*annee)&3) != 0)))
  {
    *jour = 1;
    (*mois)++;
    if (*mois > 12)
    {
      *mois = 1;
      (*annee)++;
    }
  }
}

int nb_calculs = 0;
unsigned long int duree_calculs = 0;

void testerEphemeride(int jour, int mois, int annee, int nbjours,
                      int longitude_ouest_degres, int longitude_ouest_minutes, int longitude_ouest_secondes,
                      int latitude_nord_degres, int latitude_nord_minutes, int latitude_nord_secondes)
{
  double longitude_ouest, latitude_nord;
  double lever, meridien, coucher;
  int i;
  unsigned long int duree;

  afficherTexte("\nCalcul ephemeride au lieu ");
  afficherCoordonnee(longitude_ouest_degres, longitude_ouest_minutes, longitude_ouest_secondes, 'W');
  afficherTexte(" - ");
  afficherCoordonnee(latitude_nord_degres, latitude_nord_minutes, latitude_nord_secondes, 'N');
  afficherTexte("\n\nDate            Lever         Meridien      Coucher\n");

  longitude_ouest = calculerCoordonneeDecimale(longitude_ouest_degres, longitude_ouest_minutes, longitude_ouest_secondes);
  latitude_nord = calculerCoordonneeDecimale(latitude_nord_degres, latitude_nord_minutes, latitude_nord_secondes);

  for (i=0; i<nbjours; i++)
  {
    afficherDate(jour, mois, annee);
    afficherTexte(" ");

    duree = millis();
    calculerEphemeride(jour, mois, annee, longitude_ouest, latitude_nord, &lever, &meridien, &coucher);
    duree = millis() - duree;
    duree_calculs += duree;
    nb_calculs++;

    afficherHeure(lever);
    afficherTexte(" ");
    afficherHeure(meridien);
    afficherTexte(" ");
    afficherHeure(coucher);
    afficherTexte("\n");

    avancerDate(&jour, &mois, &annee);
  }
}

void setup()
{
  char texte[80];

  initialiserAffichage();


/////////////////////////////////////////////////////////////////////////////////////////////////
//// test here!!! modify the line N° 317
  ///////////////////////////////////////////////////////////////////////////////////////////////
  

 
  testerEphemeride(1, 11, 2014, 30, -2, 17, 40, 48, 51, 28);

 //testerEphemeride(day,month,year,number of days , long ° , long min , long sec , Lat ° , lat min , lat sec);


//////////////////////////////////////////////////////////////////////////////////////////////////

  //Ephéméride à l'opéra de Sidney, sur 10 jours à compter du 1er décembre 2014
  testerEphemeride(1, 12, 2014, 10, -151, 12, 54, -33, 51, 25);

  //Ephéméride au pont Golden Gate de San Francisco, sur les 2 premières semaines de 2015
  testerEphemeride(1, 1, 2015, 14, 122, 28, 39, 37, 48, 50);

  //durée moyenne de calcul
  duree_calculs = duree_calculs * 1000 / nb_calculs;
  sprintf(texte,"\nDuree moyenne de calcul d'une ephemeride = %ld microsecondes\n", duree_calculs);
  afficherTexte(texte);
}

void loop()
{
}
